@@ -7,16 +7,19 @@ use log::debug;
 use log::LevelFilter;
 use std::env;
 
+use crate::cli::CliHarness;
+
 mod cli;
+mod collector;
 mod config;
 mod display;
-mod error;
-mod report;
+mod repository_view;
 mod run;
 mod status;
 
-/// Initializes the logger based on the debug flag and `RUST_LOG` environment variable and calls
-/// [`cli::parse_and_run()`] to generate a [`config::Config`] and eventually call [`run::run()`].
+/// Initializes the logger based on the debug flag and `RUST_LOG` environment variable and uses
+/// the [`CliHarness`] to generate a [`Config`](config::Config). Then, this calls
+/// [`CliHarness::run()`].
 fn main() -> anyhow::Result<()> {
     match env::var("RUST_LOG").is_err() {
         true => Builder::new().filter_level(LevelFilter::Off).init(),
@@ -24,7 +27,8 @@ fn main() -> anyhow::Result<()> {
     }
     debug!("initialized logger");
 
-    cli::parse_and_run()?;
+    let cli_harness = CliHarness::new();
+    cli_harness.run()?;
     Ok(())
 }
 
@@ -32,29 +36,28 @@ fn main() -> anyhow::Result<()> {
 mod tests {
     use super::*;
 
-    use crate::config::{ColorMode, Config, DisplayMode};
-    use crate::report::{LabeledReports, Report};
-    use crate::status::Status;
-
-    use anyhow::{anyhow, Result};
-
+    use anyhow::anyhow;
     use git2::ErrorCode;
     use git2::Oid;
     use git2::Repository;
     use git2::Signature;
-
     use pretty_assertions::assert_eq;
-    use std::collections::BTreeMap;
     use std::fs::File;
     use std::path::{Path, PathBuf};
     use std::{fs, io};
     use tempfile::tempdir;
 
+    use crate::collector::{RepositoryCollection, RepositoryCollector};
+    use crate::config::{ColorMode, Config, DisplayMode};
+    use crate::repository_view::RepositoryView;
+    use crate::run::RunHarness;
+    use crate::status::Status;
+
     /// This integration test for `gfold` covers an end-to-end usage scenario. It uses the
     /// [`tempfile`](tempfile) crate to create some repositories with varying states and levels
     /// of nesting.
     #[test]
-    fn integration() -> Result<()> {
+    fn integration() -> anyhow::Result<()> {
         env_logger::builder()
             .is_test(true)
             .filter_level(LevelFilter::Info)
@@ -128,76 +131,107 @@ mod tests {
         let mut config = Config::try_config_default()?;
         config.path = root.path().to_path_buf();
         config.color_mode = ColorMode::Never;
-        run::run(&config)?;
+        let run_harness = RunHarness::new(&config);
+        run_harness.run()?;
 
-        // Now, let's ensure our reports are what we expect.
-        let mut expected_reports: LabeledReports = BTreeMap::new();
-        let expected_reports_key = root
+        // Now, let's run a second time, but generate the collection directly and ensure the
+        // resulting views match what we expect.
+        let mut expected_collection = RepositoryCollection::new();
+        let expected_views_key = root
             .path()
             .to_str()
             .ok_or_else(|| anyhow!("could not convert PathBuf to &str"))?
             .to_string();
-        let mut expected_reports_raw = vec![
-            Report::new(&repo_one, "HEAD", &Status::Unclean, None, None)?,
-            Report::new(&repo_two, "HEAD", &Status::Clean, None, None)?,
-            Report::new(&repo_three, "HEAD", &Status::Clean, None, None)?,
+        let mut expected_views = vec![
+            RepositoryView::finalize(
+                &repo_one,
+                Some("HEAD".to_string()),
+                Status::Unclean,
+                None,
+                None,
+                Vec::with_capacity(0),
+            )?,
+            RepositoryView::finalize(
+                &repo_two,
+                Some("HEAD".to_string()),
+                Status::Clean,
+                None,
+                None,
+                Vec::with_capacity(0),
+            )?,
+            RepositoryView::finalize(
+                &repo_three,
+                Some("HEAD".to_string()),
+                Status::Clean,
+                None,
+                None,
+                Vec::with_capacity(0),
+            )?,
         ];
-        expected_reports_raw.sort_by(|a, b| a.name.cmp(&b.name));
-        expected_reports.insert(Some(expected_reports_key), expected_reports_raw);
+        expected_views.sort_by(|a, b| a.name.cmp(&b.name));
+        expected_collection.insert(Some(expected_views_key), expected_views);
 
-        // Add nested reports to the expected reports map.
-        let nested_expected_reports_key = nested
+        // Add nested views to the expected collection.
+        let nested_expected_views_key = nested
             .to_str()
             .ok_or_else(|| anyhow!("could not convert PathBuf to &str"))?
             .to_string();
-        let mut nested_expected_reports_raw = vec![
-            Report::new(
+        let mut nested_expected_views_raw = vec![
+            RepositoryView::finalize(
                 &repo_four,
-                "HEAD",
-                &Status::Clean,
+                Some("HEAD".to_string()),
+                Status::Clean,
                 Some("https://github.com/nickgerace/gfold".to_string()),
                 None,
+                Vec::with_capacity(0),
             )?,
-            Report::new(&repo_five, "HEAD", &Status::Unclean, None, None)?,
-            Report::new(
+            RepositoryView::finalize(
+                &repo_five,
+                Some("HEAD".to_string()),
+                Status::Unclean,
+                None,
+                None,
+                Vec::with_capacity(0),
+            )?,
+            RepositoryView::finalize(
                 &repo_six,
-                "master",
-                &Status::Unpushed,
+                Some("master".to_string()),
+                Status::Unpushed,
                 Some("https://github.com/nickgerace/gfold".to_string()),
                 None,
+                Vec::with_capacity(0),
             )?,
-            Report::new(
+            RepositoryView::finalize(
                 &repo_seven,
-                "needtopush",
-                &Status::Unpushed,
+                Some("needtopush".to_string()),
+                Status::Unpushed,
                 Some("https://github.com/nickgerace/gfold".to_string()),
                 None,
+                Vec::with_capacity(0),
             )?,
         ];
-        nested_expected_reports_raw.sort_by(|a, b| a.name.cmp(&b.name));
-        expected_reports.insert(
-            Some(nested_expected_reports_key),
-            nested_expected_reports_raw,
-        );
+        nested_expected_views_raw.sort_by(|a, b| a.name.cmp(&b.name));
+        expected_collection.insert(Some(nested_expected_views_key), nested_expected_views_raw);
 
-        // Use classic display mode to avoid collecting email results.
+        // Generate a collection. Use classic display mode to avoid collecting email results.
         config.display_mode = DisplayMode::Classic;
-        let found_labeled_reports = report::generate_reports(&config.path, &config.display_mode)?;
-        let mut found_labeled_reports_sorted = LabeledReports::new();
-        for labeled_report in found_labeled_reports {
-            let mut value = labeled_report.1;
-            value.sort_by(|a, b| a.name.cmp(&b.name));
-            found_labeled_reports_sorted.insert(labeled_report.0.clone(), value.clone());
-        }
+        let found_collection = RepositoryCollector::run(&config.path, config.display_mode)?;
 
+        // Ensure the found collection matches our expected one. Sort the collection for the
+        // assertion.
+        let mut found_collection_sorted = RepositoryCollection::new();
+        for (key, mut value) in found_collection {
+            value.sort_by(|a, b| a.name.cmp(&b.name));
+            found_collection_sorted.insert(key, value);
+        }
         assert_eq!(
-            expected_reports,             // expected
-            found_labeled_reports_sorted  // actual
+            expected_collection,     // expected
+            found_collection_sorted  // actual
         );
         Ok(())
     }
 
-    fn create_directory<P: AsRef<Path>>(parent: P, name: &str) -> Result<PathBuf> {
+    fn create_directory<P: AsRef<Path>>(parent: P, name: &str) -> anyhow::Result<PathBuf> {
         let parent = parent.as_ref();
         let new_directory = parent.join(name);
 
@@ -213,13 +247,13 @@ mod tests {
         Ok(new_directory)
     }
 
-    fn create_file<P: AsRef<Path>>(parent: P) -> Result<()> {
+    fn create_file<P: AsRef<Path>>(parent: P) -> anyhow::Result<()> {
         let parent = parent.as_ref();
         File::create(parent.join("file"))?;
         Ok(())
     }
 
-    fn commit_head_and_create_branch(repository: &Repository, name: &str) -> Result<()> {
+    fn commit_head_and_create_branch(repository: &Repository, name: &str) -> anyhow::Result<()> {
         // We need to commit at least once before branching.
         let commit_oid = commit(repository, "HEAD")?;
         let commit = repository.find_commit(commit_oid)?;
@@ -228,7 +262,7 @@ mod tests {
     }
 
     // Source: https://github.com/rust-lang/git2-rs/pull/885
-    fn commit(repository: &Repository, update_ref: &str) -> Result<Oid> {
+    fn commit(repository: &Repository, update_ref: &str) -> anyhow::Result<Oid> {
         // We will commit the contents of the index.
         let mut index = repository.index()?;
         let tree_oid = index.write_tree()?;
